@@ -354,7 +354,7 @@ Parse_set * mk_parse_set(fast_matcher_t *mchxt,
 	Pset_bucket *xt;
 	count_t *count;
 
-	assert(null_count < 0x7fff, "mk_parse_set() called with null_count < 0.");
+	assert(null_count < 0x7fff, "Called with null_count < 0.");
 
 	count = table_lookup(ctxt, lw, rw, le, re, null_count, NULL);
 
