@@ -45,11 +45,21 @@
    The implementation uses probed hashing (i.e. not bucket).
  */
 
+#define STR_POOL
+#define MEM_POOL_INIT (8*1024)
+#define MEM_POOL_INCR (16*1024)
+struct str_mem_pool_s
+{
+	str_mem_pool *prev;
+	size_t size;
+	char block[0];
+};
+
 static unsigned int hash_string(const char *str, const String_set *ss)
 {
 	unsigned int accum = 0;
 	for (;*str != '\0'; str++)
-		accum = (7 * accum) + (unsigned char)*str;
+		accum = (139 * accum) + (unsigned char)*str;
 	return accum;
 }
 
@@ -113,6 +123,7 @@ String_set * string_set_create(void)
 	ss->count = 0;
 	ss->string_pool = NULL;
 	ss_pool_alloc(MEM_POOL_INIT, ss);
+	ss->available_count = MAX_STRING_SET_TABLE_SIZE(ss->size);
 
 	return ss;
 }
@@ -138,7 +149,7 @@ static unsigned int find_place(const char *str, unsigned int h, String_set *ss)
 	while (!place_found(str, &ss->table[key], h, ss))
 	{
 		key += 2 * ++coll_num - 1;
-		if (key >= ss->size) key -= ss->size;
+		if (key >= ss->size) key = ss->mod_func(key);
 	}
 
 	return key;
@@ -164,6 +175,8 @@ static void grow_table(String_set *ss)
 			ss->table[p] = old.table[i];
 		}
 	}
+	ss->available_count = MAX_STRING_SET_TABLE_SIZE(ss->size);
+
 	/* printf("growing from %zu to %zu\n", old.size, ss->size); */
 	free(old.table);
 }
@@ -192,11 +205,10 @@ const char * string_set_add(const char * source_string, String_set * ss)
 	ss->table[p].str = str;
 	ss->table[p].hash = h;
 	ss->count++;
+	ss->available_count--;
 
-	/* We just added it to the table.  If the table got too big,
-	 * we grow it.  Too big is defined as being more than 3/8 full.
-	 * There's a huge boost from keeping this sparse. */
-	if ((8 * ss->count) > (3 * ss->size)) grow_table(ss);
+	/* We just added it to the table. */
+	if (ss->available_count == 0) grow_table(ss); /* Too full */
 
 	return str;
 }
